@@ -59,6 +59,8 @@ import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.context.ContextService;
 import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.log.audit.AuditLogService;
+import com.openexchange.mail.api.AuthenticationFailedHandlerService;
+import com.openexchange.mail.oauth.MailOAuthService;
 import com.openexchange.mail.transport.TransportProvider;
 import com.openexchange.mail.transport.config.NoReplyConfigFactory;
 import com.openexchange.mail.transport.listener.MailTransportListener;
@@ -111,6 +113,8 @@ public final class SMTPActivator extends HousekeepingActivator {
             trackService(NoReplyConfigFactory.class);
             trackService(AuditLogService.class);
             trackService(OAuthService.class);
+            trackService(AuthenticationFailedHandlerService.class);
+            trackService(MailOAuthService.class);
             track(MailcapCommandMap.class, new MailcapServiceTracker(context));
             openTrackers();
 
@@ -130,7 +134,7 @@ public final class SMTPActivator extends HousekeepingActivator {
     public void stopBundle() throws Exception {
         try {
             ListenerChain.releaseInstance();
-            cleanUp();
+            super.stopBundle();
             Services.setServiceLookup(null);
         } catch (final Throwable t) {
             LOG.error("", t);
